"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Determines if the given value is a `ModuleDefinition` object,
 * or a string that's shorthand for `ModuleDefinition.moduleId`.
 * @internal
 */
function isModuleDefinition(value) {
    return value && typeof value.moduleId === "string";
}
exports.isModuleDefinition = isModuleDefinition;
/**
 * Determines if the given value is a `Plugin` object.
 * @internal
 */
function isPlugin(value) {
    let plugin = value;
    // tslint:disable-next-line: cyclomatic-complexity
    return Boolean(plugin &&
        typeof plugin === "object" &&
        (
        // A plugin must implement at least one method
        plugin.initialize || plugin.processFile || plugin.processFiles ||
            plugin.read || plugin.watch || plugin.clean || plugin.dispose)
        &&
            (plugin.processFile === undefined ||
                typeof plugin.processFile === "function" ||
                typeof plugin.processFile === "string" ||
                isModuleDefinition(plugin.processFile))) &&
        (plugin.initialize === undefined || typeof plugin.initialize === "function") &&
        (plugin.processFiles === undefined || typeof plugin.processFiles === "function") &&
        (plugin.read === undefined || typeof plugin.read === "function") &&
        (plugin.watch === undefined || typeof plugin.watch === "function") &&
        (plugin.clean === undefined || typeof plugin.clean === "function") &&
        (plugin.dispose === undefined || typeof plugin.dispose === "function");
}
exports.isPlugin = isPlugin;
/**
 * Determines whether the given Plugin implements the `read()` method
 * @internal
 */
function isFileSourcePlugin(plugin) {
    return !!plugin.read;
}
exports.isFileSourcePlugin = isFileSourcePlugin;
/**
 * Determines whether the given Plugin implements the `processFile()` and/or `processFiles()` methods.
 * @internal
 */
function isFileProcessorPlugin(plugin) {
    return !!plugin.processFile || !!plugin.processFiles;
}
exports.isFileProcessorPlugin = isFileProcessorPlugin;
/**
 * Determines whether the given Plugin implements the `watch()` method
 * @internal
 */
function isWatcherPlugin(plugin) {
    return !!plugin.watch;
}
exports.isWatcherPlugin = isWatcherPlugin;
/**
 * Determines whether the given Plugin implements the `clean()` method
 * @internal
 */
function isCleanerPlugin(plugin) {
    return !!plugin.clean;
}
exports.isCleanerPlugin = isCleanerPlugin;
/**
 * Determines whether the given Plugin implements the `dispose()` method
 * @internal
 */
function isDisposablePlugin(plugin) {
    return !!plugin.dispose;
}
exports.isDisposablePlugin = isDisposablePlugin;
//# sourceMappingURL=types.js.map